/*
 * Written by Volker Augustin <snowbird@bigfoot.de> (03/2001)
 */

#include "cocoa_document.h"
#include "global.h"
#include <qapplication.h>
#include <qregexp.h>
#include <qmessagebox.h>

CocoaDocument::CocoaDocument(const QString& _fileName, CocoaOutput *_engine, CocoaOutput *_cocoaOutput, \
	QWidget *parent, const char *name)
	: CompletingMultiLineEdit(parent, name), fileName(_fileName), engine(_engine), cocoaOutput(_cocoaOutput)
{
	clearAfterExecute = false;
	auto_indenting = false;

	setWordWrap(QtMultiLineEdit::FixedPixelWidth);
	setWrapColumnOrWidth(SHRT_MAX);

	/* The next line is a work around for a Qt bug */
	setTableFlags(Tbl_autoHScrollBar);

	setWrapPolicy(Anywhere);
	/* setAutoUpdate(true); */
	updateTableSize();

	keywordList << "Block" << "Catch" << "Define" << "For";
	keywordList << "Foreach" << "If" << "Repeat" << "Using" << "While";
}

bool CocoaDocument::textMarked() const
{
	return hasMarkedText();
}

void CocoaDocument::execInput(ButtonState buttonState)
{
	if (engine)
	{
		if (engine->isBlocked())
		{
			QMessageBox::critical(this, QString(PROGRAM_NAME) + " " + XCOCOA_VERSION
				+ " - Error Handler",
				"The CoCoA engine has been stopped. To continue your work,\n"
				"you must restart CoCoA!",
				QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton);
		} else if (!engine->isExecuting())
		{
			bool textMarked = hasMarkedText();
			if (!textMarked)
				selectAll();
			QString s = markedText();
			if (!textMarked)
				deselect();
			if (s.isEmpty())
				return;
			else
			{
				if (clearAfterExecute)
					clear();
				emit execute(s, textMarked, buttonState);
			}
		}
	}
}

void CocoaDocument::setFileName(const QString& _fileName)
{
	fileName = _fileName;
}

const QString& CocoaDocument::getFileName() const
{
	return fileName;
}

void CocoaDocument::setAutoIndenting(bool b)
{
	auto_indenting = b;
}

void CocoaDocument::keyPressEvent(QKeyEvent *e)
{
	if ((e->state() & KeyButtonMask) && (e->key() == Key_Return))
		execInput(e->state());
	else if ((e->key() == Key_Return) && auto_indenting)
	{
		/*
		 * This code block implements AutoIndentation for CoCoA and has not been
		 * thoroughly tested. Therefore, it is deactivated by default and cannot
		 * be activated in the GUI. The only way to activate this feature is to
		 * insert the following entry in the GUI configuration file:
		 *
		 * autoIndentation=1
		 *
		 */
		int line, col;
		getCursorPosition(&line, &col);
		QString *text = getString(line);

		QRegExp startExp1("[^\\t]");
		QRegExp startExp2("\\s");

		int start1 = text->find(startExp1);
		int start2 = text->find(startExp2, start1 + 1);

		if (start2 == -1)
			start2 = text->length();

		QString startText = QString::null;
		if (start1 >= 0 && start2 > start1)
			startText = text->mid(start1, start2 - start1);

		QRegExp endExp1("\\sEnd[a-zA-Z]*;\\s*$");
		QRegExp endExp2(";\\s*$");

		int end1 = text->findRev(endExp1);
		int end2 = text->findRev(endExp2);

		QString endText = QString::null;
		if (end1 >= 0 && end2 > end1)
			endText = text->mid(end1 + 1, end2 - end1 - 1);

		CompletingMultiLineEdit::keyPressEvent(e);

		int depth;
		if (start1 != -1)
			depth = start1;
		else
			depth = text->length();

		if (keywordList.contains(startText) && !((endText == "End")|| (endText == "End" + startText)))
		{
			depth++;
		} else if ((
				(startText == endText + ";")
				&& ((endText == "End") || ((endText.length() > 3) && keywordList.contains(endText.mid(3))))
			) || (
				(startText == "In") && ((endText == "End") || (endText == "EndCatch"))
			))
		{
			depth--;
			if (text->startsWith("\t"))
			{
				getCursorPosition(&line, &col);
				setCursorPosition(line - 1, 0);
				del();
				setCursorPosition(line, col);
			}
		}

		QRegExp tabExp("[^\\t]");
		getCursorPosition(&line, &col);
		int start3 = getString(line)->find(tabExp);
		if (start3 > 0)
			depth -= start3;
		QString insertion;
		insertion.fill('\t', depth);
		insert(insertion);
	} else if ((e->state() == ControlButton) && (e->key() == Key_Insert))
	{
		/*
		 * Workaround for Qt bug on Windows
		 */
		emit toggleAutoCompletionMode();
	} else if ((e->state() & AltButton) && cocoaOutput)
	{
		switch (e->key()) {
			case Key_A:
			{
				QKeyEvent *f = new QKeyEvent(QEvent::KeyPress, e->key(), e->ascii(), e->state() ^ AltButton, e->text(), e->isAutoRepeat(), e->count());
				qApp->postEvent(cocoaOutput, f);
				break;
			}
			case Key_Up:
			case Key_Down:
			case Key_PageDown:
			case Key_PageUp:
			{
				QKeyEvent *f = new QKeyEvent(QEvent::KeyPress, e->key(), e->ascii(), e->state() ^ AltButton, e->text(), e->isAutoRepeat(), e->count());
				qApp->postEvent(cocoaOutput, f);
				break;
			}
			case Key_Home:
			case Key_End:
			{
				QKeyEvent *f = new QKeyEvent(QEvent::KeyPress, e->key(), e->ascii(), ((e->state() ^ AltButton) | ControlButton), e->text(), e->isAutoRepeat(), e->count());
				qApp->postEvent(cocoaOutput, f);
				break;
			}
			case Key_Delete:
			{
				cocoaOutput->clear();
				break;
			}
			default:
			{
				CompletingMultiLineEdit::keyPressEvent(e);
				break;
			}
		}
	} else
		CompletingMultiLineEdit::keyPressEvent(e);
}

void CocoaDocument::wheelEvent(QWheelEvent *e)
{
	if ((e->state() == AltButton) && cocoaOutput)
	{
		QWheelEvent *f = new QWheelEvent(e->pos(), e->globalPos(), e->delta(), e->state() ^ AltButton);
		qApp->postEvent(cocoaOutput, f);
	} else
		CompletingMultiLineEdit::wheelEvent(e);
}

void CocoaDocument::setClearAfterExecute(bool b)
{
	clearAfterExecute = b;
}

#include "cocoa_document.moc"
