/*
 * Written by Volker Augustin <snowbird@bigfoot.de> (03/2001)
 */

#include "interactive_document.h"
#include <iostream.h>

InteractiveDocument::InteractiveDocument(CocoaOutput *_engine, CocoaOutput *_cocoaOutput, QWidget *parent, const char *name)
	: CocoaDocument(QString::null, _engine, _cocoaOutput, parent, name)
{
	pageIndex = 0;
	historyEnabled = true;

	emit previousPageAvailable(false);
	emit nextPageAvailable(true);
}

int InteractiveDocument::getPageIndex()
{
	return pageIndex;
}

bool InteractiveDocument::previousPageAvailable()
{
	return ((int) pageList.count() > 0) && (pageIndex > 0);
}

bool InteractiveDocument::nextPageAvailable()
{
	return historyEnabled;
}

void InteractiveDocument::activeStateChanged(bool active)
{
	historyEnabled = active;
	emit previousPageAvailable(active && previousPageAvailable());
	emit nextPageAvailable(active);
	emit isActive(active);
}

void InteractiveDocument::keyPressEvent(QKeyEvent *e)
{
	if (e->state() & AltButton)
	{
		switch (e->key())
		{
			case Key_Left:
			{
				previousPage();
				break;
			}
			case Key_Right:
			{
				nextPage();
				break;
			}
			default:
				CocoaDocument::keyPressEvent(e); break;
		}
        } else if ((e->key() == Key_Return) && (e->state() == ShiftButton)
		&& ((int) pageList.count() > 0) && (pageIndex < (int) pageList.count()))
	{
		pageList[pageIndex] = text();
	} else if ((e->state() == ControlButton) && (e->key() == Key_Delete))
	{
		deleteCurrentHistoryPage();
	} else
		CocoaDocument::keyPressEvent(e);
}


void InteractiveDocument::deleteCurrentHistoryPage()
{
	if (((int) pageList.count() == 0) || (pageIndex == (int) pageList.count()))
	{
		if (!text().isEmpty())
		{
			leaveCompletionMode();
			clear();
			clearUndoHistory();
		}
	} else if (pageIndex < (int) pageList.count() - 1)
	{
		leaveCompletionMode();
		pageList.remove(pageList.at(pageIndex));
		setText(pageList[pageIndex]);
		clearUndoHistory();
	} else if (pageIndex == (int) pageList.count() - 1)
	{
		leaveCompletionMode();
		pageList.remove(pageList.at(pageIndex));
		setText(editPage);
		editPage = "";
		clearUndoHistory();
	}
}


void InteractiveDocument::clearHistory()
{
	pageList.clear();
	pageIndex = pageList.count();
	emit previousPageAvailable(false);
	emit pageIndexChanged(pageIndex);
}


/*
 * Workaround for missing feature in Qt:
 * The only way to clear the undo/redo history is to disable and reenable it.
 */
void InteractiveDocument::clearUndoHistory()
{
	bool state = isUndoEnabled();
	setUndoEnabled(false);
	setUndoEnabled(state);
}

void InteractiveDocument::appendPage(QString commands, bool interactiveMode, bool jumpToEnd)
{
	if (((int) pageList.count() == 0) || (pageList[pageList.count() - 1] != commands))
	{
		if ((pageList.count() > 0) && (pageIndex == (int) pageList.count()))
			pageIndex++;
		else if ((int) pageList.count() == 0)
			pageIndex = 1;
		pageList.append(commands);
	}

	if (jumpToEnd)
	{
		leaveCompletionMode();
		setText(editPage);
		editPage = "";
		pageIndex = pageList.count();
	}

	emit previousPageAvailable(previousPageAvailable());
	emit pageIndexChanged(pageIndex);
	if (interactiveMode)
		emitCursorPosition();
}

void InteractiveDocument::nextPage()
{
	if (((int) pageList.count() == 0) || (pageIndex == (int) pageList.count()))
	{
		if (!text().isEmpty())
		{
			leaveCompletionMode();
			appendPage(text(), true, false);
			clear();
			clearUndoHistory();
		}
	} else if (pageIndex < (int) pageList.count() - 1)
	{
		leaveCompletionMode();
		pageIndex++;
		setText(pageList[pageIndex]);
		clearUndoHistory();
	} else if (pageIndex == (int) pageList.count() - 1)
	{
		leaveCompletionMode();
		pageIndex++;
		setText(editPage);
		editPage = "";
		clearUndoHistory();
	}
	emit previousPageAvailable(previousPageAvailable());
	emit pageIndexChanged(pageIndex);

	emitCursorPosition();
}

void InteractiveDocument::previousPage()
{
	if (((int) pageList.count() > 0) && (pageIndex > 0))
	{
		leaveCompletionMode();
		/* There do exist pages and we are not currently showing the fist one */
		if (pageIndex == (int) pageList.count())
		{
			/*
			 * Since we are currently showing the "active page", we have to
			 * save it temporarily
			 */
			editPage = text();
			pageIndex = pageList.count() - 1;
			setText(pageList[pageIndex]);
		} else
		{
			pageIndex--;
			setText(pageList[pageIndex]);
		}
		clearUndoHistory();
	}
	emit previousPageAvailable(previousPageAvailable());
	emit pageIndexChanged(pageIndex);

	emitCursorPosition();
}

#include "interactive_document.moc"
