/*
 * Written by Volker Augustin, snowbird@bigfoot.de (03/2001)
 */

#include "cocoa_output.h"
#include "main.h"
#include "menu_item_enabler.h"
#include "pixmaps.h"
#include <qfiledialog.h>
#include <qwhatsthis.h>

#if defined(MAKE_KDE)
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kapp.h>
#else
#include <qmenubar.h>
#include <qapplication.h>
#endif

#include <qmessagebox.h>
#include <qpaintdevicemetrics.h>
#include <qpainter.h>

#include <stdlib.h>

#if defined(SIS_WIN32)
	#include <windows.h>
#endif

int ca_init(void);
extern "C" char * ca_application_path;
extern "C" char * ca_cocoa_packages();
extern "C" char * ca_getenv(const char *name);

const char* fileNewText = "Create a new CoCoA document.<br>"
		"You can also select <b>New</b> from the <b>File</b> menu.";
const char* fileOpenText = "Open an existing file.<br>"
		"You can also select <b>Open...</b> from the <b>File</b> menu.";
const char* fileCloseText = "Close the current document. If the document contains unsaved changes "
		"you will be prompted if you want to save it.<br>"
		"You can also select <b>Close</b> from the <b>File</b> menu.";
const char* fileSaveText = "Save the current document. If it is untitled you will be prompted for a file name.<br>"
		"You can also select <b>Save</b> from the <b>File</b> menu.";
const char* filePrintText = "Print the current document.<br>"
		"You can also select <b>Print</b> from the <b>File</b> menu.";

const char* editUndoText = "Undo the last text operation";
const char* editRedoText = "Redo the last text operation";
const char* editCutText = "Cut selected text";
const char* editCopyText = "Copy selected text";
const char* editPasteText = "Paste selected text";
const char* editSelectAllText = "Select all text from the text widget that has the focus.<b>"
		"If no text widget has the focus, no text is selected.";

const char* cocoaPreviousText = "Load the previous interactive page into the editor";
const char* cocoaNextText = "Load the next interactive page into the editor";
const char* cocoaExecuteText = "Execute the current command set";
const char* cocoaInterruptText = "Interrupt the current computation";
const char* cocoaClearOutputText = "Clear the output window";

const char* outputWindowText = "CoCoA output appears here";
const char* inputWindowText = "Enter your CoCoA commands here";

CocoaOutput *cocoaOut = NULL;

ApplicationWindow::ApplicationWindow(QDir _applicationPath)
#if defined(MAKE_KDE)
	: KMainWindow(0, QString(PROGRAM_NAME) + " application main window", WDestructiveClose),
		applicationPath(_applicationPath)
#else
	: QMainWindow(0, QString(PROGRAM_NAME) + " application main window", WDestructiveClose),
		applicationPath(_applicationPath)
#endif
{
	/* Determine package path */
	char *cocoaLibraryEnvVar = ca_cocoa_packages();
	QString libraryPathName;
	if (cocoaLibraryEnvVar && strlen(cocoaLibraryEnvVar) > 0 && strcmp(cocoaLibraryEnvVar, "."))
		libraryPath = QDir(cocoaLibraryEnvVar);
	else
		libraryPath = applicationPath;

	
	printer = new QPrinter;
	clipboard = QApplication::clipboard();

	/*
	 * Workaround for a KDE bug:
	 * We have to initialize the statusbar before we add anything else to
	 * the main window. Otherwise, the statusbar will not be shown.
	 */
	statusBar();
	statusBar()->setSizeGripEnabled(false);
#if defined(MAKE_KDE)
	messageBar = new KStatusBar(statusBar());
#else
	messageBar = new QStatusBar(statusBar());
#endif
	messageBar->setSizeGripEnabled(false);
	statusBar()->addWidget(messageBar, 1, true);


	/* Load the GUI properties */
	char *propertyEnvVar = ca_getenv("COCOA_GUI_PROPERTIES");

	if (propertyEnvVar && strlen(propertyEnvVar) > 0)
		propertyFileName = propertyEnvVar;
	else
		propertyFileName = UI_CONFIG_FILE;

	loadProperties();
	loadDefaultProperties();


	bool activate_auto_indentation = false;
	if (getAppProperty(APP_AUTO_INDENTATION) != QString::null)
		activate_auto_indentation = true;


	splitter = new QSplitter(QSplitter::Vertical, this);
	output = new CocoaOutput(splitter, "CoCoA Output Class");
	output->setMinimumHeight(60);
	cocoaOut = output;
	input    = new CocoaInput(output, output, splitter, "Input Class");

	/*
	 * FIXME: What is the new syntax?
	 * QWhatsThis::add(input, inputWindowText);
	 * QWhatsThis::add(output, outputWindowText);
	 */

	QIconSet newIconSet = createIconSet(filenew_xpm);
	QIconSet openIconSet = createIconSet(fileopen_xpm);
	QIconSet saveIconSet = createIconSet(filesave_xpm);
	QIconSet closeIconSet = createIconSet(fileclose_xpm);
	QIconSet printIconSet = createIconSet(fileprint_xpm);
	QIconSet exitIconSet = createIconSet(exit_xpm);

	QIconSet undoIconSet = createIconSet(undo_xpm);
	QIconSet redoIconSet = createIconSet(redo_xpm);
	QIconSet cutIconSet = createIconSet(editcut_xpm);
	QIconSet copyIconSet = createIconSet(editcopy_xpm);
	QIconSet pasteIconSet = createIconSet(editpaste_xpm);

	QIconSet leftIconSet = createIconSet(leftarrow_xpm);
	QIconSet rightIconSet = createIconSet(rightarrow_xpm);
	QIconSet execIconSet = createIconSet(exec_xpm);
	QIconSet stopIconSet = createIconSet(stop_xpm);
	QIconSet clearOutputIconSet = createIconSet(clear_output_xpm);

	QIconSet outputTopIconSet = createIconSet(output_top_xpm);
	QIconSet outputBottomIconSet = createIconSet(output_bottom_xpm);
	QIconSet fontIconSet = createIconSet(text_xpm);
	QIconSet configureIconSet = createIconSet(configure_xpm);

	QIconSet contentsIconSet = createIconSet(contents_xpm);
	QIconSet contextHelpIconSet = createIconSet(contexthelp_xpm);
	QPixmap cocoaIcon = QPixmap(cocoa_xpm);

#ifndef SIS_WIN32
	setIcon(cocoaIcon);
#endif

#if defined(MAKE_KDE)
	fileTools = new KToolBar(this, "file operations");
	fileTools->setFullSize(false);
#else
	fileTools = new QToolBar(this, "file operations");
#endif
	addToolBar(fileTools);

	QToolButton *fileNewButton = new QToolButton(newIconSet, "New File", QString::null,
		input, SLOT(createDocument()), fileTools, "new file");
	QToolButton *fileOpenButton = new QToolButton(openIconSet, "Open File", QString::null,
		input, SLOT(load()), fileTools, "open file");
	QToolButton *fileCloseButton = new QToolButton(closeIconSet, "Close File", QString::null,
		this, SLOT(closeWindow()), fileTools, "close file");
	connect(input, SIGNAL(closeAvailable(bool)), fileCloseButton, SLOT(setEnabled(bool)));
	QToolButton *fileSaveButton = new QToolButton(saveIconSet, "Save File", QString::null,
		input, SLOT(save()), fileTools, "save file");
	connect(input, SIGNAL(saveAvailable(bool)), fileSaveButton, SLOT(setEnabled(bool)));
	QToolButton *filePrintButton = new QToolButton(printIconSet, "Print File", QString::null,
		this, SLOT(print()), fileTools, "print file");
	fileTools->addSeparator();
	QToolButton *undoButton = new QToolButton(undoIconSet, "Undo last operation", QString::null,
		input, SLOT(undo()), fileTools, "undo last operation");
	connect(input, SIGNAL(undoAvailable(bool)), undoButton, SLOT(setEnabled(bool)));
	QToolButton *redoButton = new QToolButton(redoIconSet, "Redo last operation", QString::null,
		input, SLOT(redo()), fileTools, "redo last operation");
	connect(input, SIGNAL(redoAvailable(bool)), redoButton, SLOT(setEnabled(bool)));
	QToolButton *cutButton = new QToolButton(cutIconSet, "Cut", QString::null,
		input, SLOT(cut()), fileTools, "cut");
	connect(input, SIGNAL(copyAvailable(bool)), cutButton, SLOT(setEnabled(bool)));
	QToolButton *copyButton = new QToolButton(copyIconSet, "Copy", QString::null,
		input, SLOT(copy()), fileTools, "copy");
	connect(input, SIGNAL(copyAvailable(bool)), copyButton, SLOT(setEnabled(bool)));
	pasteButton = new QToolButton(pasteIconSet, "Paste", QString::null,
		input, SLOT(paste()), fileTools, "paste");
	pasteButton->setEnabled(clipboard->text());
	fileTools->addSeparator();

	(void) QWhatsThis::whatsThisButton(fileTools);

	QWhatsThis::add(fileNewButton, fileNewText);
	QWhatsThis::add(fileOpenButton, fileOpenText);
	QWhatsThis::add(fileCloseButton, fileCloseText);
	QWhatsThis::add(fileSaveButton, fileSaveText);
	QWhatsThis::add(filePrintButton, filePrintText);

	QWhatsThis::add(undoButton, editUndoText);
	QWhatsThis::add(redoButton, editRedoText);
	QWhatsThis::add(cutButton, editCutText);
	QWhatsThis::add(copyButton, editCopyText);
	QWhatsThis::add(pasteButton, editPasteText);


	/* menu 'CoCoA' */
#if defined(MAKE_KDE)
	cocoaTools = new KToolBar(this, "CoCoA operations");
	cocoaTools->setFullSize(false);
#else
	cocoaTools = new QToolBar(this, "CoCoA operations");
#endif
	addToolBar(cocoaTools);

	QToolButton *leftButton = new QToolButton(leftIconSet, "Previous input", QString::null,
		input->interactiveDocument, SLOT(previousPage()), cocoaTools, "previous input");
	leftButton->setEnabled(input->interactiveDocument->previousPageAvailable());
	connect(input->interactiveDocument, SIGNAL(previousPageAvailable(bool)), leftButton, SLOT(setEnabled(bool)));
	QToolButton *rightButton = new QToolButton(rightIconSet, "Next input", QString::null,
		input->interactiveDocument, SLOT(nextPage()), cocoaTools, "next input");
	leftButton->setEnabled(input->interactiveDocument->nextPageAvailable());
	connect(input->interactiveDocument, SIGNAL(nextPageAvailable(bool)), rightButton, SLOT(setEnabled(bool)));
	QToolButton *execButton = new QToolButton(execIconSet, "Execute current command set", QString::null,
		input, SLOT(execute()), cocoaTools, "execute current command set");
	/* HACK */
	connect(this, SIGNAL(execButtonEnabled(bool)), execButton, SLOT(setEnabled(bool)));
	/* END OF HACK */
	QToolButton *stopButton = new QToolButton(stopIconSet, "Interrupt current computation", QString::null,
		this, SLOT(interruptComputation()), cocoaTools, "interrupt current computation");
	connect(output, SIGNAL(executing(bool)), stopButton, SLOT(setEnabled(bool)));
	QToolButton *clearOutputButton = new QToolButton(clearOutputIconSet, "Clear output window", QString::null,
		output, SLOT(clear()), cocoaTools, "clear output window");

	QWhatsThis::add(leftButton, cocoaPreviousText);
	QWhatsThis::add(rightButton, cocoaNextText);
	QWhatsThis::add(execButton, cocoaExecuteText);
	QWhatsThis::add(stopButton, cocoaInterruptText);
	QWhatsThis::add(clearOutputButton, cocoaClearOutputText);


	/* menu 'File' */
	fileMenu = new QPopupMenu(this);
	fileMenu->setCheckable(false);
	menuBar()->insertItem("&File", fileMenu);
	fileNewId     = fileMenu->insertItem(newIconSet, "&New", input, SLOT(createDocument()), CTRL + Key_N);
	fileOpenId    = fileMenu->insertItem(openIconSet, "&Open...", input, SLOT(load()), CTRL + Key_O);
	fileCloseId  = fileMenu->insertItem(closeIconSet, "&Close", this, SLOT(closeWindow()), CTRL + Key_W);
	fileMenu->insertSeparator();
	fileSaveId    = fileMenu->insertItem(saveIconSet, "&Save", input, SLOT(save()), CTRL + Key_S);
	fileSaveAsId  = fileMenu->insertItem("Save &as...", input, SLOT(saveAs()));
	fileSaveOutputId = fileMenu->insertItem("Save Outpu&t As...", this, SLOT(saveOutputAs()));
	fileMenu->insertSeparator();
	idFilePrint   = fileMenu->insertItem(printIconSet, "&Print...", this, SLOT(print()), CTRL + Key_P);
	fileMenu->insertSeparator();
	fileMenu->insertItem(exitIconSet, "&Quit", qApp, SLOT(closeAllWindows()), CTRL + Key_Q);

	fileMenu->setWhatsThis(fileOpenId, fileOpenText );
	fileMenu->setWhatsThis(fileSaveId, fileSaveText );
	fileMenu->setWhatsThis(fileSaveAsId, fileSaveText);
	fileMenu->setWhatsThis(idFilePrint, filePrintText );

	fileMenu->setItemEnabled(fileSaveId, false);
	fileMenu->setItemEnabled(fileSaveAsId, true);
	fileMenu->setItemEnabled(fileCloseId, false);


	/* menu 'Edit' */
	editMenu = new QPopupMenu(this);
	menuBar()->insertItem("&Edit", editMenu);
	undoId      = editMenu->insertItem(undoIconSet, "&Undo", input, SLOT(undo()), CTRL + Key_Z);
	redoId      = editMenu->insertItem(redoIconSet, "&Redo", input, SLOT(redo()), CTRL + Key_Y);
	editMenu->insertSeparator();
	cutId       = editMenu->insertItem(cutIconSet, "C&ut",  input, SLOT(cut()), CTRL + Key_X);
	copyId      = editMenu->insertItem(copyIconSet, "&Copy", input, SLOT(copy()), CTRL + Key_C);
	pasteId     = editMenu->insertItem(pasteIconSet, "&Paste", input, SLOT(paste()), CTRL + Key_V);
	selectAllId = editMenu->insertItem("Select all", this, SLOT(selectAll()), CTRL + Key_A);

	editMenu->setWhatsThis(undoId, editUndoText);
	editMenu->setWhatsThis(redoId, editRedoText);
	editMenu->setWhatsThis(cutId, editCutText);
	editMenu->setWhatsThis(copyId, editCopyText);
	editMenu->setWhatsThis(pasteId, editPasteText);
	editMenu->setWhatsThis(selectAllId, editSelectAllText);

	/*
	 * Normally we would have to check for the current status of all these things,
	 * but it should be okay to assume, that there is no text in the current editor,
	 * so we cannot redo, undo, copy or cut anything.
	 */
	editMenu->setItemEnabled(undoId, false);
	editMenu->setItemEnabled(redoId, false);
	editMenu->setItemEnabled(cutId, false);
	editMenu->setItemEnabled(copyId, false);
	editMenu->setItemEnabled(pasteId, clipboard->text());

	connect(input, SIGNAL(closeAvailable(bool)), this, SLOT(setCloseItemEnabled(bool)));
	connect(input, SIGNAL(copyAvailable(bool)), this, SLOT(setCopyCutItemEnabled(bool)));
	connect(input, SIGNAL(redoAvailable(bool)), this, SLOT(setRedoItemEnabled(bool)));
	connect(input, SIGNAL(undoAvailable(bool)), this, SLOT(setUndoItemEnabled(bool)));
	connect(input, SIGNAL(saveAvailable(bool)), this, SLOT(setSaveItemEnabled(bool)));


	/* menu 'CoCoA' */
	cocoaMenu = new QPopupMenu(this);
	cocoaMenu->setCheckable(true);
	menuBar()->insertItem("&CoCoA", cocoaMenu);

	previousPageId = cocoaMenu->insertItem(leftIconSet, "&Previous interactive page", input->interactiveDocument, SLOT(previousPage()), ALT + Key_Left);
	nextPageId     = cocoaMenu->insertItem(rightIconSet, "&Next interactive page", input->interactiveDocument, SLOT(nextPage()), ALT + Key_Right);
	deleteCurrentHistoryPageId = cocoaMenu->insertItem("Delete current history page", input->interactiveDocument, SLOT(deleteCurrentHistoryPage()), CTRL + Key_Delete);
	clearHistoryId = cocoaMenu->insertItem("Clear input history", input->interactiveDocument, SLOT(clearHistory()));
	cocoaMenu->insertSeparator();
	executeId      = cocoaMenu->insertItem(execIconSet, "&Execute current command set", input, SLOT(execute()), CTRL + Key_Return);
	interruptId    = cocoaMenu->insertItem(stopIconSet, "Interrupt current computation", this, SLOT(interruptComputation()), ALT + Key_Pause);
	cocoaMenu->insertSeparator();
	clearOutputId  = cocoaMenu->insertItem(clearOutputIconSet, "C&lear output window", output, SLOT(clear()), ALT + Key_Delete);

	cocoaMenu->setItemEnabled(previousPageId, input->interactiveDocument->previousPageAvailable());
	cocoaMenu->setItemEnabled(nextPageId, input->interactiveDocument->nextPageAvailable());
	cocoaMenu->setItemEnabled(clearOutputId, true);

	connect(input->interactiveDocument, SIGNAL(previousPageAvailable(bool)),
		new MenuItemEnabler(cocoaMenu, previousPageId), SLOT(setItemEnabled(bool)));
	connect(input->interactiveDocument, SIGNAL(nextPageAvailable(bool)),
		new MenuItemEnabler(cocoaMenu, nextPageId), SLOT(setItemEnabled(bool)));
	connect(input->interactiveDocument, SIGNAL(isActive(bool)),
		new MenuItemEnabler(cocoaMenu, deleteCurrentHistoryPageId), SLOT(setItemEnabled(bool)));
	connect(input->interactiveDocument, SIGNAL(isActive(bool)),
		new MenuItemEnabler(cocoaMenu, clearHistoryId), SLOT(setItemEnabled(bool)));
	connect(this, SIGNAL(execButtonEnabled(bool)),
		new MenuItemEnabler(cocoaMenu, executeId), SLOT(setItemEnabled(bool)));
	connect(output, SIGNAL(executing(bool)),
		new MenuItemEnabler(cocoaMenu, interruptId), SLOT(setItemEnabled(bool)));


	/* menu 'Settings' */
	settingsMenu = new QPopupMenu( this );
	settingsMenu->setCheckable(true);
	menuBar()->insertItem("&Settings", settingsMenu);

	outputTopId      = settingsMenu->insertItem(outputTopIconSet, "Output on Top", this, SLOT(setOutputWindowPosition(int)));
	outputBottomId   = settingsMenu->insertItem(outputBottomIconSet, "Output on Bottom", this, SLOT(setOutputWindowPosition(int)));
	settingsMenu->insertSeparator();
	noWrapId         = settingsMenu->insertItem("No WordWrap", this, SLOT(setWordWrap(int)));
	fixedWrapId      = settingsMenu->insertItem("Fixed WordWrap", this, SLOT(setWordWrap(int)));
	normalWrapId     = settingsMenu->insertItem("Normal WordWrap", this, SLOT(setWordWrap(int)));
	settingsMenu->insertSeparator();
	autoCompletingId = settingsMenu->insertItem("&Autocompleting", this, SLOT(autoCompleting()), CTRL + Key_Insert);

	if (activate_auto_indentation)
		autoIndentingId  = settingsMenu->insertItem("Auto&indenting", this, SLOT(autoIndenting()));

	settingsMenu->insertSeparator();
	fontSelectId     = settingsMenu->insertItem(fontIconSet, "Select Font...", this, SLOT(selectFont()));
	configureId      = settingsMenu->insertItem(configureIconSet, "Word completion...", this, SLOT(showPreferencesDialog()));

	settingsMenu->setItemParameter(outputTopId, outputTopId);
	settingsMenu->setItemParameter(outputBottomId, outputBottomId);

	wordWrapTypes.setAutoDelete(true);
	wordWrapTypes.append(new WordWrap(noWrapId, WORD_WRAP_NONE));
	wordWrapTypes.append(new WordWrap(fixedWrapId, WORD_WRAP_FIXED));
	wordWrapTypes.append(new WordWrap(normalWrapId, WORD_WRAP_NORMAL));

	WordWrap *wrapType;
	for (wrapType = wordWrapTypes.first(); wrapType != 0; wrapType = wordWrapTypes.next())
		settingsMenu->setItemParameter(wrapType->menuIndex, wrapType->menuIndex);


	/* menu 'Help' */
	QPopupMenu *helpMenu = new QPopupMenu(this);
	menuBar()->insertSeparator();
	menuBar()->insertItem("&Help", helpMenu );

	help = new HelpWindow(libraryPath.canonicalPath() + "/doc/index.html", ".", NULL, "help viewer");
#ifndef SIS_WIN32
	help->setIcon(cocoaIcon);
#endif
	helpMenu->insertItem(contentsIconSet, "&Contents", help, SLOT(showAndRaise()), Key_F1);
	helpMenu->insertItem(contextHelpIconSet, "What's &This", this, SLOT(whatsThis()), SHIFT + Key_F1);
	helpMenu->insertSeparator();
	helpMenu->insertItem(cocoaIcon, QString("&About ") + PROGRAM_NAME + "...", this, SLOT(about()));


	connect(clipboard, SIGNAL(dataChanged()), this, SLOT(setPasteAvailable()));
	setPasteAvailable();


	cursorPositionIndicator = new QLabel(statusBar());
	statusBar()->addWidget(cursorPositionIndicator, 0, true);

	historyIndexIndicator = new QLabel(statusBar());
	setHistoryIndex(input->interactiveDocument->getPageIndex());
	statusBar()->addWidget(historyIndexIndicator, 0, true);

	wordWrapIndicator = new QLabel(statusBar());
	statusBar()->addWidget(wordWrapIndicator, 0, true);

	autoCompletionIndicator = new QLabel(statusBar());
	statusBar()->addWidget(autoCompletionIndicator, 0, true);

	autoIndentationIndicator = new QLabel(statusBar());
	if (activate_auto_indentation)
		statusBar()->addWidget(autoIndentationIndicator, 0, true);
	else
		autoIndentationIndicator->hide();


	connect(input->interactiveDocument, SIGNAL(isActive(bool)), historyIndexIndicator, SLOT(setEnabled(bool)));
	connect(input, SIGNAL(documentSelected(const QString &)),this, SLOT(documentSelected(const QString &)));
	connect(input, SIGNAL(setStatus(QString)), this, SLOT(setStatus(QString)));
	connect(input, SIGNAL(setStatus(QString, int)), this, SLOT(setStatus(QString, int)));
	connect(input->interactiveDocument, SIGNAL(pageIndexChanged(int)), this, SLOT(setHistoryIndex(int)));
	connect(input, SIGNAL(cursorPositionChanged(int, int)), this, SLOT(showCursorPosition(int, int)));
	connect(output, SIGNAL(currentStatus(QString)), this, SLOT(setStatus(QString)));
	connect(this, SIGNAL(FontSelected(const QFont&)), output, SLOT(selectFont(const QFont&)));
	connect(this, SIGNAL(FontSelected(const QFont&)), input, SLOT(setFont(const QFont&)));
	connect(this, SIGNAL(autoCompleting(bool)), input, SLOT(setAutoCompleting(bool)));
	connect(this, SIGNAL(autoIndenting(bool)), input, SLOT(setAutoIndenting(bool)));
	connect(output, SIGNAL(executing(bool)), this, SLOT(setExecutionDisabled(bool)));
	/*
	 * Workaround for Qt bug on Windows
	 */
	connect(input, SIGNAL(toggleAutoCompletionMode()), this, SLOT(autoCompleting()));

	setCentralWidget(splitter);
	input->interactiveDocument->setFocus();


	/* set the font */
	bool ok;
	int fontSize = getAppProperty(APP_FONT_SIZE).toInt(&ok);
	if (ok)
		selectFont(QFont(getAppProperty(APP_FONT_NAME), fontSize));
	else
		selectFont(QFont(getAppProperty(APP_FONT_NAME)));


	/* resize the window */
	int width = getAppProperty(APP_WINDOW_WIDTH).toInt(&ok);
	if (ok)
	{
		int height = getAppProperty(APP_WINDOW_HEIGHT).toInt(&ok);
		if (ok)
		{
			QWidget *d = QApplication::desktop();
			int screen_width = d->width();
			int screen_height = d->height();
			resize((width > screen_width ? screen_width : width), (height > screen_height ? screen_height : height));
		}
	}


	/* set the window position */
	int xpos = getAppProperty(APP_WINDOW_X).toInt(&ok);
	if (ok)
	{
		int ypos = getAppProperty(APP_WINDOW_Y).toInt(&ok);
		if (ok)
			move(xpos, ypos);
        }


	/* set the position of the toolbars */
	int toolsOrientation = getAppProperty(APP_FILE_TOOLBAR_ORIENTATION).toInt(&ok);
	if (ok)
		fileTools->setOrientation((Qt::Orientation) toolsOrientation);

	int toolsDock = getAppProperty(APP_FILE_TOOLBAR_DOCK).toInt(&ok);
	int toolsIndex = getAppProperty(APP_FILE_TOOLBAR_INDEX).toInt(&ok);
	int toolsNewLine = getAppProperty(APP_FILE_TOOLBAR_NEWLINE).toInt(&ok);
	int toolsExtraOffset = getAppProperty(APP_FILE_TOOLBAR_EXTRAOFFSET).toInt(&ok);
	moveToolBar(fileTools, (ToolBarDock) toolsDock, (bool) toolsNewLine, toolsIndex, toolsExtraOffset);

	toolsOrientation = getAppProperty(APP_COCOA_TOOLBAR_ORIENTATION).toInt(&ok);
	if (ok)
		cocoaTools->setOrientation((Qt::Orientation) toolsOrientation);
	toolsDock = getAppProperty(APP_COCOA_TOOLBAR_DOCK).toInt(&ok);
	toolsIndex = getAppProperty(APP_COCOA_TOOLBAR_INDEX).toInt(&ok);
	toolsNewLine = getAppProperty(APP_COCOA_TOOLBAR_NEWLINE).toInt(&ok);
	toolsExtraOffset = getAppProperty(APP_COCOA_TOOLBAR_EXTRAOFFSET).toInt(&ok);
	moveToolBar(cocoaTools, (ToolBarDock) toolsDock, (bool) toolsNewLine, toolsIndex, toolsExtraOffset);


	/* set word wrap mode */
	if (getAppProperty(APP_WORD_WRAP) == WORD_WRAP_FIXED) setWordWrap(fixedWrapId);
	else if (getAppProperty(APP_WORD_WRAP) == WORD_WRAP_NORMAL) setWordWrap(normalWrapId);
	else if (getAppProperty(APP_WORD_WRAP) == WORD_WRAP_NONE) setWordWrap(noWrapId);
	else setWordWrap(noWrapId);


	/* set output window position */
	if (getAppProperty(APP_OUTPUT_WINDOW_POSITION) == OWP_TOP) setOutputWindowPosition(outputTopId);
	else if (getAppProperty(APP_OUTPUT_WINDOW_POSITION) == OWP_BOTTOM) setOutputWindowPosition(outputBottomId);
	else setOutputWindowPosition(outputTopId);


	/* set window space division */
	int topSize = getAppProperty(APP_TOP_WINDOW_SIZE).toInt(&ok);
	if (ok)
	{
		int bottomSize = getAppProperty(APP_BOTTOM_WINDOW_SIZE).toInt(&ok);
		if (ok)
		{
			QValueList<int> sizes;
			sizes.append(topSize);
			sizes.append(bottomSize);
			splitter->setSizes(sizes);
		}
	}


	/* set autocompletion mode */
	if (getAppProperty(APP_AUTO_COMPLETION) == ENABLED) setAutoCompleting(true);
	else setAutoCompleting(false);


	/* Load user wordlists */
	reloadCompletionFiles();
	input->setWordCompletions(&wordCompletions);


	/* set input tabwidth */
	QString tabWidthString = getAppProperty(APP_TABWIDTH);
	if (tabWidthString != QString::null)
	{
		int tabwidth = tabWidthString.toInt(&ok);
		if (ok)
			QtMultiLineEdit::setDefaultTabStop(tabwidth);
	}


	/* set autoindentation mode */
	if (getAppProperty(APP_AUTO_INDENTATION) == ENABLED) setAutoIndenting(true);
	else setAutoIndenting(false);


	/* Load bookmarks for the help browser */
	PropertySection *helpBookmarks = properties.get(SECTION_HELP_BOOKMARKS);
	PropertySection *oldHelpBookmarks = help->setBookmarks(helpBookmarks);
	delete oldHelpBookmarks;


	/* Load history for the help browser */
	PropertySection *helpHistory = properties.get(SECTION_HELP_HISTORY);
	PropertySection *oldHelpHistory = help->setHistory(helpHistory);
	delete oldHelpHistory;
}


void ApplicationWindow::selectAll()
{
	QWidget *w = qApp->focusWidget();
	if (w->inherits("QtMultiLineEdit"))
		((QtMultiLineEdit*) w)->selectAll();
}

void ApplicationWindow::setPasteAvailable()
{
	pasteButton->setEnabled(clipboard->text());
	editMenu->setItemEnabled(pasteId, clipboard->text());
}

bool ApplicationWindow::loadFile(QString fileName)
{
	return input->load(fileName);
}

void ApplicationWindow::showCursorPosition(int line, int column)
{
	cursorPositionIndicator->setText(QString("Line: ") + QString::number(line + 1) + " Col: " + QString::number(column + 1));
}

void ApplicationWindow::setExecutionDisabled(bool b)
{
	emit execButtonEnabled(!b);
}

ApplicationWindow::~ApplicationWindow()
{
	delete printer;
}

void ApplicationWindow::interruptComputation()
{
	output->engineInterrupted(true);
}

void ApplicationWindow::setCopyCutItemEnabled(bool b)
{
	editMenu->setItemEnabled(copyId, b);
	editMenu->setItemEnabled(cutId, b);
}

void ApplicationWindow::setRedoItemEnabled(bool b) { editMenu->setItemEnabled(redoId, b); }
void ApplicationWindow::setUndoItemEnabled(bool b) { editMenu->setItemEnabled(undoId, b); }
void ApplicationWindow::setCloseItemEnabled(bool b) { fileMenu->setItemEnabled(fileCloseId, b); }
void ApplicationWindow::setSaveItemEnabled(bool b) { fileMenu->setItemEnabled(fileSaveId, b); }

void ApplicationWindow::setStatus(QString s)
{
	/* Messages will be shown forever */
	messageBar->message(s);
}

void ApplicationWindow::setStatus(QString s, int ms)
{
	messageBar->message(s, ms);
}

void ApplicationWindow::documentSelected(const QString &fileName)
{
	if (fileName == NULL)
		setCaption(QString(PROGRAM_NAME) + " " + XCOCOA_VERSION + ": New Document");
	else
		setCaption(QString(PROGRAM_NAME) + " " + XCOCOA_VERSION + ": " + fileName);
}

void ApplicationWindow::setHistoryIndex(int index)
{
	historyIndexIndicator->setText(QString("HI: ") + QString::number(index + 1));
}

void ApplicationWindow::autoCompleting()
{
	bool newstate = !settingsMenu->isItemChecked(autoCompletingId);
	setAutoCompleting(newstate);
}

void ApplicationWindow::autoIndenting()
{
	bool newstate = !settingsMenu->isItemChecked(autoIndentingId);
	setAutoIndenting(newstate);
}

bool ApplicationWindow::saveOutputAs()
{
	bool aborted = false;

	while (!aborted)
	{
		QString fileName;
		bool useFileName = false;
		do
		{
			fileName = QFileDialog::getSaveFileName(QString::null, QString::null, this, 0, "Save output window");
			if (fileName.isEmpty())
				aborted = true;

			if (!aborted)
			{				
				if (QFile(fileName).exists())
				{
					if (QMessageBox::warning(this,
						QString(PROGRAM_NAME) + " " + XCOCOA_VERSION,
						fileName + " already exists.\n"
						"Do you want to overwrite it?",
						"Yes", "No", 0, 1) == 0)
						useFileName = true;
				} else
					useFileName = true;
			}
		} while (!(aborted || useFileName));

		if (!aborted)
		{
			int success = saveAs(output->text(), fileName);
			if (success)
			{
				emit setStatus(QString("Output saved to %1").arg(fileName), 10000);
				return true;
			} else
			{
				emit setStatus(QString("Could not write to %1").arg(fileName), 10000);
				if (QMessageBox::warning(this,
					QString(PROGRAM_NAME) + " " + XCOCOA_VERSION,
					"Could not save the contents of the output window!",
					"Save as", "Cancel", 0, 0) == 1)
				{
					aborted = true;
					useFileName = false;
				}
			}
		}
	}

	emit setStatus("Saving aborted", 10000);
	return false;
}

bool ApplicationWindow::saveAs(const QString& text, const QString& fileName)
{
	QFile file(fileName);
	if (!file.open(IO_WriteOnly))
		return false;

	QTextStream out(&file);
	QIODevice *dev = out.device();
	out << text;
	if (dev->status() != IO_Ok)
		return false;
	dev->close();
	if (dev->status() != IO_Ok)
		return false;
	return true;
}

/*
 * print() written by Alessandro Polverini (2000)
 */
void ApplicationWindow::print()
{
	const int margin = 10;
	int pageNo = 1;

	if (printer->setup(this))
	{
		setStatus( "Printing..." );
		QPainter p;
		p.begin(printer);
		p.setFont(output->font());
		int yPos = 0;
		QFontMetrics fm = p.fontMetrics();
		QPaintDeviceMetrics metrics(printer);
                                                                                                                                                                                // of printer surface
		for(int i = 0 ; i < output->numLines(); i++)
		{
			if (margin + yPos > metrics.height() - margin)
			{
				setStatus(QString("Printing (page %1)...").arg(++pageNo));
				printer->newPage();
				yPos = 0;
			}
			p.drawText(margin, margin + yPos, metrics.width(), fm.lineSpacing(),
					ExpandTabs | DontClip, output->textLine(i));
			yPos = yPos + fm.lineSpacing();
		}
		p.end();
		setStatus("Printing completed");
	} else
	setStatus("Printing aborted");
}

void ApplicationWindow::closeWindow()
{
	input->closeCurrentDocument();
}

void ApplicationWindow::about()
{
	QString text = QString("Interface version ") + UI_VERSION + " by\n"
			"Volker Augustin <snowbird@bigfoot.de>";
	QString caption = QString(PROGRAM_NAME) + " " + XCOCOA_VERSION;

#if defined(MAKE_KDE)
	KMessageBox::about(this, text, caption);
#else
	QMessageBox msg(caption, text,
		QMessageBox::Information,
		QMessageBox::Ok | QMessageBox::Default,
		QMessageBox::NoButton,
		QMessageBox::NoButton,
		this);

	QPixmap bigCocoaIcon(bigcocoa_xpm);
	msg.setIconPixmap(bigCocoaIcon);

	msg.show();
#endif
}

int main(int argc, char ** argv)
{
#if defined(SIS_WIN32)
	int bufferSize = 0;
	int charNum = bufferSize - 1;
	char *pathBuffer = 0;
	while ((charNum != 0) && (charNum == bufferSize - 1))
	{
		if (pathBuffer != NULL)
			delete[] pathBuffer;
		
		bufferSize += 128;
		pathBuffer = new char[bufferSize];
		
		charNum = GetModuleFileName(NULL, pathBuffer, bufferSize - 1);
	}
	
	if (charNum)
	{
		argv[0] = new char[charNum + 1];
		strcpy(argv[0], pathBuffer);

		for (int i = charNum - 1; i >= 0; i--)
		{
			if (pathBuffer[i] == '\\')
			{
				pathBuffer[i] = '\0';
				break;
			}
		}
		ca_application_path = pathBuffer;
	}
#else
	QFileInfo fi(argv[0]);
	while (fi.isSymLink())
	{
		QFileInfo temp_fi(fi.dir(true), fi.readLink());
		QString absFilePath = temp_fi.absFilePath();
		fi = QFileInfo(absFilePath);
	}
	QString dirName = fi.dirPath(true);
	QString fileName = fi.fileName();

	ca_application_path = new char[dirName.length() + 1];
	strcpy(ca_application_path, dirName.latin1());

	argv[0] = new char[fi.absFilePath().length() + 1];
	strcpy(argv[0], fi.absFilePath().latin1());
#endif

	/*
	 * Since KApplication changes argv[0] (it strips the path) we have to
	 * initialize appDir before KApplication is initialized.
	 */
	QDir appDir = QFileInfo(argv[0]).dir(true);

#if defined(MAKE_KDE)
	KApplication a(argc, argv, "kcocoa");
#else
	QApplication a(argc, argv);
#endif
	ApplicationWindow * mw = new ApplicationWindow(appDir);
	mw->setCaption("Document 1");
	int n = ca_init();
	mw->show();

	a.connect(&a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()));

	/* Initialize the CoCoA packages */
	cocoaOut->execInput("<< CocoaPackagePath() + '/cocoa/init.coc';");

	for (int i = 1; i < argc; i++)
		mw->loadFile(QString(argv[i]));
		
	return a.exec();
}

#include "main.moc"
