/*
 * Written by Volker Augustin <snowbird@bigfoot.de> (02/2001)
 */

#include "properties.h"
#include <qfile.h>
#include <qiodevice.h>

istream& operator>>(istream& s, QString& string)
{
	char buffer[10];
	char c = 0;

	do
	{
		if (c == '\n')
			break;
		if (c)
			string += c;
		s.get(buffer, 10, '\n');
		string += buffer;
	} while (s.get(c));
	return s;
}

const QString& PropertySectionEntry::key() const
{
	return _key;
}

const QString& PropertySectionEntry::value() const
{
	return _value;
}

void PropertySectionEntry::setValue(const QString& value)
{
	_value = value;
}

ostream& operator<<(ostream& s, const PropertySectionEntry& entry)
{
	s << entry.key().latin1() << "=";
	if (entry.value() != QString::null)
		s << entry.value().latin1();
	return s;
}

istream& operator>>(istream& s, PropertySectionEntry& entry)
{
	QString string = "";
	char buffer[10];
	char c = 0;

	do
	{
		if (c == '\n')
			break;
		if (c)
			string += c;
		s.get(buffer, 10, '\n');
		string += buffer;
	} while (s.get(c));

	int equal_position = string.find('=');
	entry._key = string.left(equal_position);
	entry._value = equal_position >= 0 ? string.mid(equal_position + 1) : QString::null;
	return s;
}

QTextStream& operator<<(QTextStream& s, const PropertySectionEntry& entry)
{
	QIODevice *dev = s.device();
	if (dev->isOpen() && dev->isWritable())
	{
		s << entry.key() << "=";
		if (entry.value() != QString::null)
			s << entry.value();
	}
	return s;
}

QTextStream& operator>>(QTextStream& s, PropertySectionEntry& entry)
{
	QIODevice *dev = s.device();
	if (dev->isOpen() && dev->isReadable())
	{
		QString string = s.readLine();
		int equal_position = string.find('=');
		entry._key = string.left(equal_position);
		entry._value = equal_position >= 0 ? string.mid(equal_position + 1) : QString::null;
	}
	return s;
}

const QString& PropertySection::name() const
{
	return _name;
}

void PropertySection::append(const QString& key, const QString& value)
{
	append(new PropertySectionEntry(key, value));
}

void PropertySection::append(PropertySectionEntry* entry)
{
	((QList<PropertySectionEntry>*) this)->append(entry);
}

bool PropertySection::remove(uint index)
{
	return ((QList<PropertySectionEntry>*) this)->remove(index);
}

bool PropertySection::remove(const QString& key)
{
	QListIterator<PropertySectionEntry> it(*this);
	for (; it.current(); ++it)
	{
		PropertySectionEntry *entry = it.current();
		if (entry->key() == key)
			return ((QList<PropertySectionEntry>*) this)->remove(entry);
	}
	return false;
}

int PropertySection::compareItems(QCollection::Item item1, QCollection::Item item2)
{
	PropertySectionEntry *pItem1 = (PropertySectionEntry*) item1, *pItem2 = (PropertySectionEntry*) item2;
	return (pItem1->_key).compare(pItem2->_key);
}

uint PropertySection::contains(const QString& key) const
{
	QListIterator<PropertySectionEntry> it(*this);
	uint count = 0;
	for (; it.current(); ++it)
	{
		PropertySectionEntry *entry = it.current();
		if (entry->key() == key) count++;
	}
	return count;
}

int PropertySection::find(const QString& key)
{
	return ((QList<PropertySectionEntry>*) this)->find(new PropertySectionEntry(key, QString::null));
}

const QString& PropertySection::get(const QString& key)
{
	int index = find(key);
	if (index == -1)
		return QString::null;
	else
		return at(index)->value();
}

bool PropertySection::set(const QString& key, const QString& value)
{
	int index = find(key);
	if (index != -1)
	{
		at(index)->setValue(value);
		return true;
	} else
	{
		PropertySectionEntry *entry = new PropertySectionEntry(key, value);
		append(entry);
		return false;
	}
}

ostream& operator<<(ostream& s, const PropertySection& section)
{
	QListIterator<PropertySectionEntry> it(section);
	s << "[" << section.name().latin1() << "]" << NewLine;
	for (; it.current(); ++it)
		s << *(it.current()) << NewLine;
	return s;
}

istream& operator>>(istream& s, PropertySection& section)
{
	PropertySectionEntry *entry;
	char c = 0;
	while (s.get(c) && c != '\n')
	{
		s.putback(c);
		entry = new PropertySectionEntry();
		s >> *entry;

		/* If we already have an entry with the same key remove it */
		int position = section.find(entry->key());
		if (position != -1)
			section.remove(position);

		section.append(entry);
	}

	return s;
}

QTextStream& operator<<(QTextStream& s, const PropertySection& section)
{
	QIODevice *dev = s.device();
	if (dev->isOpen() && dev->isWritable())
	{
		QListIterator<PropertySectionEntry> it(section);
		s << "[" << section.name() << "]" << NewLine;
		for (; it.current(); ++it)
			s << *(it.current()) << NewLine;
	}
	return s;
}

QTextStream& operator>>(QTextStream& s, PropertySection& section)
{
	QIODevice *dev = s.device();
	if (dev->isOpen() && dev->isReadable())
	{
		PropertySectionEntry *entry;
		QString line;
		while (!s.atEnd() && (line = s.readLine()) && !line.isEmpty())
		{
			QTextIStream istream(&line);
			entry = new PropertySectionEntry();
			istream >> *entry;

			/* If we already have an entry with the same key remove it */
			int position = section.find(entry->key());
			if (position != -1)
				section.remove(position);

			section.append(entry);
		}
	}
	return s;
}

PropertySection* PropertyFile::get(const QString& key)
{
	PropertySection* section = find(key);
	if (!section)
	{
		section = new PropertySection(key);
		insert(key, section);
	}
	return section;
}

ostream& operator<<(ostream& s, const PropertyFile& file)
{
	QDictIterator<PropertySection> it(file);
	while (it.current())
	{
		PropertySection *section = it.current();
		s << *section << NewLine;
		++it;
	}
	return s;
}

istream& operator>>(istream& s, PropertyFile& file)
{
	PropertySection *section;
	char c = 0;
	while (s.get(c) && c != '\n')
	{
		QString sectionName = QString::null;

		if (c == '[')
		{
			s >> sectionName;
			if (sectionName.right(1) == "]")
				sectionName = sectionName.mid(0, sectionName.length() - 1);
		} else s.putback(c);

		if (sectionName != QString::null)
		{
			section = file.find(sectionName);
			if (!section)
			{
				section = new PropertySection(sectionName);
				file.insert(sectionName, section);
			}
			s >> *section;
		}
	}
	return s;
}

QTextStream& operator<<(QTextStream& s, const PropertyFile& file)
{
	QIODevice *dev = s.device();
	if (dev->isOpen() && dev->isWritable())
	{
		QDictIterator<PropertySection> it(file);
		while (it.current())
		{
			PropertySection *section = it.current();
			s << *section << NewLine;
			++it;
		}
	}
	return s;
}

QTextStream& operator>>(QTextStream& s, PropertyFile& file)
{
	QIODevice *dev = s.device();
	if (dev->isOpen() && dev->isReadable())
	{
		PropertySection *section;
		QString line;
		while (!s.atEnd())
		{
			line = s.readLine();
			QString sectionName = QString::null;

			if (line.left(1) == "[")
			{
				sectionName = line.mid(1);
				if (sectionName.right(1) == "]")
					sectionName = sectionName.mid(0, sectionName.length() - 1);
			}

			if (sectionName != QString::null)
			{
				section = file.find(sectionName);
				if (!section)
				{
					section = new PropertySection(sectionName);
					file.insert(sectionName, section);
				}
				s >> *section;
			}
		}
	}
	return s;
}
